/*autor:  Marc-Antoine Drouin
  email: drouim@iro.umontreal.ca

  description: dynamic programing  with visibility computation
  
  Copyright (c) 2005 Marc-Antoine Drouin


This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#include <stdio.h>
#include <stdlib.h>
#include "dynamic.h"


/*#define DEBUG_DYNAMIC_PROG  */

/* macro facilitant l'acces au tableau */
#define GETTABLEVAL( dd , xxx , yyy ) ((dd).table[ (xxx)* NBLABEL + (yyy) ].val )
#define GETTABLEPREV( dd ,xxx , yyy ) ((dd).table[ (xxx)* NBLABEL + (yyy) ].prev )
#define GETTABLEOCC( dd , xxx , yyy ) ((dd).table[ (xxx)* NBLABEL + (yyy) ].occ )
#define GETADR(dd, xxx, yyy ) ( (xxx)* NBLABEL + (yyy)  )
#define YADR( pos )( pos % NBLABEL )




/* init structure for optimisation */
int createDynamicProg(struct dynamicProg * d,int xsize   )
{
  d->table = ( struct dynamicInt *) malloc(sizeof( struct dynamicInt)*xsize*NBLABEL);
  if (d->table == NULL)
      return -1;
  d->XSize = xsize;
  d->bestPos = -1;  




  return 0;
}



int freeDynamicProg(struct dynamicProg *d)
{
  free(d->table);
  d->table = NULL;
  return 0;  
}











void OptimizeDynamicProg(struct dynamicProg* d, const int mode,void * v  )
{
  short i,j,k;
    OPT_ACC_TYPE tmp;
    OPT_ACC_TYPE indexVal;
  short index;
  int ii,iii;
#ifdef KEEP_VISIBILITY
  MASK_TYPE mask;
  MASK_TYPE tmpmask;
#endif


#ifdef DEBUG_DYNAMIC_PROG  
  printf("Begin Dynamic Programming Optimisation...\n");
#endif  
  /* initialisation */
  for(j=0;j<NBLABEL;j++)
    {
      d->table[j].val = 0;
#ifdef KEEP_VISIBILITY
	if(mode==0)
	  indexVal = SmoothCapacityX(-1,0,0,j,0,0,&mask ,v); /* X */
	else
	  indexVal = SmoothCapacityY(-1,0,0,j,0,0,&mask,v); /* Y */
#else
	if(mode==0)
	  indexVal = SmoothCapacityX(-1,0,0,j,0,v) ; /* X */
	else
	  indexVal = SmoothCapacityY(-1,0,0,j,0,v) ; /* Y */

#endif


      d->table[j].prev =-1;
      d->table[j].occ =j;        
      d->table[j].mask =0;        
#ifdef DEBUG_DYNAMIC_PROG        
      printf("(0 %d)(%d)=%f  ",j,-1, GETTABLEVAL((*d) ,0,j ) );
#endif 
    }
#ifdef DEBUG_DYNAMIC_PROG  
  printf("\n");
#endif
  ii=0;
  iii=NBLABEL;
  for(i=1;i<d->XSize;i++)
    {     
    for(j=0;j<NBLABEL;j++)
      {

	int occ;
#ifdef KEEP_VISIBILITY
	MASK_TYPE oldmask;
	oldmask = d->table[ ii ].mask;
#endif
	occ =d->table[ ii ].occ -i;
	index = ii;

#ifdef KEEP_VISIBILITY
	if(mode==0)
	  indexVal = SmoothCapacityX(i-1,0,i,j,occ,oldmask,&mask ,v)+   d->table[ ii ].val ; /* X */
	else
	  indexVal = SmoothCapacityY(i-1,0,i,j,occ,oldmask,&mask,v)+   d->table[ ii ].val ; /* Y */
#else
	if(mode==0)
	  indexVal = SmoothCapacityX(i-1,0,i,j,occ,v)+   d->table[ ii ].val ; /* X */
	else
	  indexVal = SmoothCapacityY(i-1,0,i,j,occ,v)+   d->table[ ii ].val ; /* Y */

#endif
	for(k=1;k<NBLABEL;k++)
	  {
	    occ = d->table[ ii+k ].occ-i;
	    oldmask = d->table[ ii+k ].mask;
#ifdef KEEP_VISIBILITY
	    if(mode==0)
	      tmp =   SmoothCapacityX(i-1,k,i,j,occ,oldmask,&tmpmask,v)+ d->table[ ii+k ].val ; /* X */
	    else
	      tmp =   SmoothCapacityY(i-1,k,i,j,occ,oldmask,&tmpmask,v)+ d->table[ ii+k ].val ; /* Y */
#else
    if(mode==0)
	      tmp =   SmoothCapacityX(i-1,k,i,j,occ,v)+ d->table[ ii+k ].val ; /* X */
	    else
	      tmp =   SmoothCapacityY(i-1,k,i,j,occ,v)+ d->table[ ii+k ].val ; /* Y */
#endif
	    if (tmp < indexVal)
	      {
#ifdef KEEP_VISIBILITY
		mask=tmpmask;
#endif
		index    =ii+k;
		indexVal =tmp;
	      }
	  }
	/*fixe value and previous  */
	 d->table[iii].val    =indexVal;  
	 d->table[iii].prev   =index;

#ifdef KEEP_VISIBILITY
	 /* we save udes mask if required to do so */
	 d->table[iii].mask    =mask ;  
#endif  
	/* we update with the most occlusion doner */
	if( d->table[index].occ < j+i  )
	  d->table[iii].occ     = j+i;
	else
	  d->table[iii].occ = d->table[index].occ;
#ifdef DEBUG_DYNAMIC_PROG  
	printf("(%d %d)(%d)=%f,%f  ",i,j,index,indexVal,cache );
#endif 	
	iii++;
      }
    ii+=NBLABEL;
#ifdef DEBUG_DYNAMIC_PROG  
    printf("\n");
#endif     
    }

  /* localize best final configuration */
  index = GETADR(*d,d->XSize -1,0);
  indexVal = 	GETTABLEVAL((*d) ,d->XSize-1,0 );
  for(j=1;j<NBLABEL;j++)
    {
      tmp = 	GETTABLEVAL((*d) ,d->XSize-1,j );
      if(tmp < indexVal)
	{
	  index = GETADR(*d,d->XSize-1,j  );
	  indexVal = tmp;
	}
    }
  d->bestPos = index; 		/* we put the final solution for easy extractraction */

#ifdef DEBUG_DYNAMIC_PROG  
  printf("End Dynamic Programming Optimisation\n");
#endif  
}










/* extract minimum cut associate with minimum energy */
void ResultDynamicProg(struct dynamicProg* d,unsigned char *t, int size )
{
  short i;
  short prev;

  prev = d->bestPos;
  t[size-1] =  YADR(prev);
  for(i=size-2;i>=0;i--)
    {
      prev = d->table[prev].prev;
      t[i] =YADR(prev); 
    }
}

#ifdef KEEP_VISIBILITY
/* extract minimum cut associate with minimum energy */
void ResultVisibilityDynamicProg(struct dynamicProg* d,
				 MASK_TYPE *t, 
				 int size )
{
  short i;
  short prev;
  prev = d->bestPos;
  t[size-1] =  d->table[prev].mask;
  for(i=size-2;i>=0;i--)
    {
      prev=d->table[prev].prev;
      t[i]=d->table[prev].mask; 
    }
}
#endif
