

/*
Description: This is a general purpose implentation of a dynamic
             programing algorithm. This implentation should be thread
             safe and multiple dynamicProg object can be simultaniously used.

Normal calling order is:

createDynamicProg()
OptimizeDynamicProg()
ResultDynamicProg()
freeDynamicProg()

Copyright (C) 2005 Marc-Antoine Drouin 
                   drouim@iro.umontreal.ca 
Lab Vision 3D, DIRO Universite de Montreal 
date:   11-12-2005


 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "const.h"


#ifndef DYNAMIC_H
#define DYNAMIC_H
struct dynamicInt {
  OPT_ACC_TYPE  val;			/*value of cost  */
  int occ;			/* nb pixel occlusion */
  short prev;			/* previous element of solution*/
#ifdef KEEP_VISIBILITY
  MASK_TYPE mask;		/* mask of visibility used */
#endif

};

struct dynamicProg {
  struct dynamicInt * table; 	/* DP table */

     short XSize,			/* length of a line */
       YSize;		/* number of label for each element of an optimisation line */
  /* generic cost function to be used */
  short bestPos;			/* -1 before optimisation
				   otherwise index of the best solution for the initial
				   element of the table*/
				  
};





/* this allow to allocate resource and define the dynamic programing problem */
int createDynamicProg(struct dynamicProg* d,
		      int xsize /* length of the line */
		  
		   
		        );


/* do the optimisation of the problem define in <d> */
void OptimizeDynamicProg(struct dynamicProg* d, int mode ,void * v  );



/*  extract the argument of the resulting minimum solution*/
void ResultDynamicProg(struct dynamicProg* d,unsigned char *t, int size );



#ifdef KEEP_VISIBILITY
/*  extract the visibility argument of the resulting minimum solution*/
void ResultVisibilityDynamicProg(struct dynamicProg* d,MASK_TYPE *t, int size );
#endif


/* free used resource  */
int freeDynamicProg(struct dynamicProg* d);





#endif
